#!/bin/bash -
#===========================================================================
#
#          FILE: muttonchop_deps.sh
#
#         USAGE: ./muttonchop_deps.sh
#
#   DESCRIPTION: installs dependencies for muttonchop, gets source
#   code, builds program, installs program, config files, and
#   debian init script. USE AT YOUR OWN RISK! READ THE SCRIPT
#   CAREFULLY AND DECIDE IF YOU WANT TO USE IT
#
#       OPTIONS:
#  REQUIREMENTS: debian-based system, sudo privileges
#          BUGS: ---
#         NOTES: ---
#        AUTHOR: Jonathan Kulp ()
#       CREATED: 04/09/2013 06:03:38 AM CDT
#      REVISION:  ---
#===========================================================================

user=$(whoami)
builddir="/home/$user/muttonchop"
configdir="/home/$user/.config/muttonchop"

install_deps(){
  buildtools="bzr valac cmake build-essential"
  libs="gstreamer0.10-plugins-ugly gstreamer0.10-alsa gstreamer0.10-plugins-bad gstreamer0.10-plugins-good libjson-glib-1.0-0 libjson-glib-dev gstreamer0.10-ffmpeg libgstreamer0.10-dev libsqlite3-dev libgstreamer-plugins-base0.10-dev libgee-dev libxml2-dev"
  sudo apt-get update
  sudo apt-get install $buildtools $libs
}

get_n_build(){
  cd /home/$user
  bzr branch lp:muttonchop
  cd $builddir
  cmake ./ -DVIDEO=OFF && make && sudo make install
}

conf(){
  mkdir -p $configdir
  cat $builddir/data/muttonchop.conf | sed -e "s/user/$(whoami)/g" > $configdir/muttonchop.conf
  cp -R $builddir/data/public/ $configdir/
}

# install init script, update links

init_install(){
  sudo cp $builddir/scripts/muttonchop.init /etc/init.d/muttonchop
  sudo chmod +x /etc/init.d/muttonchop
  sudo update-rc.d muttonchop defaults
  sudo /etc/init.d/muttonchop start
}

install_deps
get_n_build
conf
init_install

exit 0
